////////////////////////////////////////////////////////////////////////
// Class:       SupernovaTrigger
// Module Type: filter
// File:        SupernovaTrigger_module.cc
//
// Generated at Mon Dec 17 15:29:53 2012 by Zukai Wang using artmod
// from art v1_02_06.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDFilter.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Persistency/Common/Assns.h"
#include "art/Framework/Principal/Handle.h"	
#include "art/Framework/Principal/Event.h"

#include "DDTBaseDataProducts/BaseProducts.h"
#include "DDTBaseDataProducts/Boundary.h"
#include "DDTBaseDataProducts/BoundaryList.h"
#include "DDTBaseDataProducts/HitList.h"
#include "DDTBaseDataProducts/DAQHit.h"
#include "DDTBaseDataProducts/GroupedHitList.h"
#include "DDTBaseDataProducts/TriggerDecision.h"	

#include <string>
#include <vector>
#include <algorithm>

namespace novaddt {
  class SupernovaTrigger;
}

class novaddt::SupernovaTrigger : public art::EDFilter {
public:
  explicit SupernovaTrigger(fhicl::ParameterSet const & p);
  virtual ~SupernovaTrigger();

  virtual bool filter(art::Event & e) override;

private:
  unsigned _prescale;
  std::string _hitslabel;
  std::string _hitsinstance;
  std::string _slicelabel;
  std::string _sliceinstance;
  std::string _tracklabel;
  std::string _trackinstance;

  float _peak_ratethr;
  float _tail_ratethr;
  float _timethr;
  float _peaktimethr;

  unsigned _trigger_counts; 
};

novaddt::SupernovaTrigger::SupernovaTrigger(fhicl::ParameterSet const & p)
  :
  _prescale(p.get<unsigned>("prescale")),
  _hitslabel (p.get< std::string >("hits_label")),  
  _hitsinstance (p.get< std::string >("hits_instance"  )),
  _slicelabel(p.get<std::string>("slice_label")),
  _sliceinstance(p.get<std::string>("slice_instance")),
  _tracklabel(p.get<std::string>("track_label")),
  _trackinstance(p.get<std::string>("track_instance")),
  _peak_ratethr(p.get<float>("peak_rate_thr")),
  _tail_ratethr(p.get<float>("tail_rate_thr")),
  _timethr(p.get<float>("time_thr")),
  _peaktimethr(p.get<float>("peak_time_thr")),
  _trigger_counts(0)

{
  // Call appropriate Produces<>() functions here.
  produces<std::vector<TriggerDecision>>("TriggerDecisions");
}

novaddt::SupernovaTrigger::~SupernovaTrigger()
{
  // Clean up dynamic memory and other resources here.
}

bool novaddt::SupernovaTrigger::filter(art::Event & e)	
{

  bool pre_result = false;
  bool result = false;
  std::unique_ptr<std::vector<TriggerDecision>> 
    trigger_decisions(new std::vector<TriggerDecision>);

  art::Handle<GroupedHitList<TDC>> slices;
  e.getByLabel(_slicelabel, _sliceinstance, slices);
  unsigned n_slices = slices->n_groups();

  //First, loop over all time slices to find any SN peak, if there is not any, just return false
  for (unsigned slice = 0; slice != n_slices; ++slice)
    {
      HitList slicehits = slices->get(slice);
      TDC t_begin = slicehits.front().TDC();
      TDC t_end = slicehits.back().TDC();

      float duration =t_end.val-t_begin.val; 
      if (duration<_peaktimethr) continue;
      else if ((slices->group_size(slice))/duration  > _peak_ratethr) pre_result = true;

    }

  if (!pre_result) return false;

  /*
    if ((slices->group_size(slice))/duration  > _ratethr)
    {
      // This event contains at least one useful slice, so let's keep it around:
      result = true; 
    }
  */

  //Now, we have to take care of the entire hitlist
  art::Handle<novaddt::HitList> tothits;
  e.getByLabel(_hitslabel,_hitsinstance,tothits);

  std::cout<<"Oh, we might see something~"<<std::endl;


  TDC Tlow   = tothits->front().TDC();
  TDC Thigh  = tothits->back().TDC();
  int n_hits = tothits->size();
  //Attention: It is not a typo of the following initialization. Don't be fooled.
  double t_low = Thigh.val;
  double t_high = Tlow.val;
  if (n_hits < _tail_ratethr * (t_low-t_high)) return false;

  art::Handle<std::vector<HitList> > tracks;
  e.getByLabel(_tracklabel, _trackinstance, tracks);

  bool flag =false;

  for (auto const& hit : *tothits)
    {
      //Loop over all tracks to get rid of hits in any track:
      for (unsigned i = 0; i!= tracks->size(); ++i) {
	flag = false;
	if (std::find((*tracks)[i].begin(), (*tracks)[i].end(), hit) != (*tracks)[i].end() )
	  {
	    flag = true;
	    break;
	  }
      }
      if (!flag) {
	double time = hit.TDC().val;
	if (time > t_high) t_high = time;
	if (time < t_low) t_low = time;
      }

      else {
	n_hits--;
	continue;
      }
    }

  //Do a rate calculation again:
  if (n_hits > _tail_ratethr * (t_high-t_low)) {
    /*
    trigger_decisions->
      emplace_back(Tlow, Thigh - Tlow, DDT_Supernova);
    */
    ++_trigger_counts;
    if (_trigger_counts%_prescale == _prescale-1) {
      trigger_decisions->
	emplace_back(Tlow, Thigh - Tlow, daqdataformats::trigID::TRIG_ID_DATA_SN,_prescale);

      result = true;
    }
  }

  e.put(std::move(trigger_decisions), "TriggerDecisions");
  return result;	
}

DEFINE_ART_MODULE(novaddt::SupernovaTrigger)
